@@ -1,8 +1,5 @@
 package Authen::NZRealMe::ServiceProvider::Builder;
-{
-  $Authen::NZRealMe::ServiceProvider::Builder::VERSION = '1.16';
-}
-
+$Authen::NZRealMe::ServiceProvider::Builder::VERSION = '1.17';
 use warnings;
 use strict;
 use feature "switch";
@@ -41,14 +38,6 @@ EOF
 
     contact_company => <<EOF,
 Which company name should be listed in the technical contact details?
-EOF
-
-    contact_first_name => <<EOF,
-What is the first name of the technical contact person?
-EOF
-
-    contact_surname => <<EOF,
-What is the first name of the technical contact person?
 EOF
 
 );
@@ -354,7 +343,7 @@ See L<Authen::NZRealMe> for documentation index.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2010-2014 Enrolment Services, New Zealand Electoral Commission
+Copyright (c) 2010-2019 Enrolment Services, New Zealand Electoral Commission
 
 Written by Grant McLean E<lt>grant@catalyst.net.nzE<gt>
 
